/*! \example testasterx1.cpp

In this example almost all driver functionalities are tested.

\b Usage:

USB Default case (just one device which uses ttyACM connected)
\code
./testasterx1
\endcode

USB any port
\code
./testasterx1 [-u DEVICEPORT (/dev/ttyACM0)] [-n NUMITERATIONS]
\endcode
 */

#include <stdlib.h>
#include <asterx1_gps.h>

using namespace std;

int main(int argc, char **argv)
{
	int opt;
	int ii;
	int retValue;
	int nIterations = 10;
	int rate = MSEC500; //ONCHANGE, MSEC20, ... , MSEC100, ... SEC1, SEC5
	string serial_device="/dev/ttyACM0";
	streamsize tsDigits;
	CasteRx1 *myGps;
  
	cout << "\n\n    TEST AsteRx1 GPS RECEIVER     \n\n" << endl;

	//cout << "acqPeriodValues[ONCHANGE]: " << acqPeriodValues[ONCHANGE] << endl;
	
	// Argument management
	while ((opt = getopt(argc, argv, "u:n:r:h?"))!= -1)
	{
		switch (opt)
		{
			case 'u': // change USB device
				serial_device = optarg;
				break;
			case 'n':
				nIterations = atoi(optarg);
				break;
			case 'r':
				rate = atoi(optarg);
				break;
			case '?': // help
			case 'h':
			default:
				cout << "  USAGE" << endl
				<< "  " << argv[0] << " [options]" << endl
				<< "  OPTIONS" << endl
				<< "   -u SERIAL_DEVICE (default: " << serial_device << ")" << endl
				<< "   -n NUMBER OF ITERATIONS (default: " << nIterations << ")" << endl
				<< "   -r ACQUISITION RATE: (default: " << rate << ")" << endl;
				for (ii=0; ii<10; ii++)
				{
					cout << "		" << ii << ": " << acqPeriodValues[ii] << endl;
				}
				cout << endl;
				return 1;
		}
	}
	
	myGps = new CasteRx1(serial_device,rate);

	myGps->setMapOrigin(41.388595, 2.113133, 120, 44.2); //sets map origin for coordinate transformation
	//myGps->printTM(); //prints to stdout the transformation matrix

	retValue = myGps->openDevice();//open device comm's
	
	retValue = myGps->closeDevice();//close device comm's (stressing tests ...)
	retValue = myGps->openDevice();//open device comm's, again
	
	myGps->setPortName("/dev/ttyS1"); //checking that when device is open, port configuration can't be achieved
	myGps->setAcquisitionRate(56); //checking that when invalid rate indexes prdouces an error and don't change the rate
	myGps->setAcquisitionRate(SEC2); //checking that when device is open rate configuration can be achieved
	myGps->setAcquisitionRate(rate); //checking that when device is open rate configuration can be achieved
	
	if ( retValue == BASIC_SUCCESS )
	{
		retValue = myGps->startAcquisition();//start data acquisition 
		retValue = myGps->stopAcquisition();//stop data acquisition (stressing tests ...)
		retValue = myGps->startAcquisition();//start data acquisition, again
		if ( retValue == BASIC_SUCCESS )
		{
			for (ii=0; ii<nIterations; ii++)
			{
				cout << endl;
				myGps->readDataFromDevice();
				//myGps->printData(cout); //prints to cout all current data in a single row. (cout can be repolaced by another ostream object reference)
				cout << "Status = " << myGps->getStatus() << endl;
				tsDigits=cout.precision(12);
				cout << "TimeStamp = " << myGps->getTimeStamp() << endl;
				cout.precision(tsDigits);
				cout << "PVT error = " << myGps->getPVTerror() << endl;
				cout << "TOW = " << myGps->getTow() << endl;
				
				if ( myGps->getStatus() == ALL_OK )
				{
					cout << "Num Of Satellites = " << myGps->getNumSatellites() << endl;
					tsDigits=cout.precision(12);
					cout << "lat = " << myGps->getLat(inDEGREES) << endl;
					cout << "lon = " << myGps->getLon(inDEGREES) << endl;
					cout.precision(tsDigits);
					cout << "alt = " << myGps->getAlt() << endl;
					cout << "xWgs = " << myGps->getXWgs() << endl;
					cout << "yWgs = " << myGps->getYWgs() << endl;
					cout << "zWgs = " << myGps->getZWgs() << endl;
					cout << "xMap = " << myGps->getXMap() << endl;
					cout << "yMap = " << myGps->getYMap() << endl;
					cout << "zMap = " << myGps->getZMap() << endl;
					cout << "vxMap = " << myGps->getVxMap() << endl;
					cout << "vyMap = " << myGps->getVyMap() << endl;
					cout << "vzMap = " << myGps->getVzMap() << endl;
					cout << "PDOP = " << myGps->getPDOP() << endl;
				}
			}
			
			retValue = myGps->stopAcquisition();
		}
		
		retValue = myGps->closeDevice();
	}

	delete myGps;
	
	return 0;
}
